package org.colinvella.fancyfish.entity;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.entity.fish.BlennyEntity;
import org.colinvella.fancyfish.entity.fish.ClownFishEntity;
import org.colinvella.fancyfish.entity.fish.DamselFishEntity;
import org.colinvella.fancyfish.entity.fish.FishEntity;
import org.colinvella.fancyfish.entity.fish.MoonFishEntity;
import org.colinvella.fancyfish.entity.fish.MoorishIdolEntity;
import org.colinvella.fancyfish.entity.fish.PufferFishEntity;
import org.colinvella.fancyfish.entity.fish.RegalTangEntity;
import org.colinvella.fancyfish.entity.fish.RoyalGrammaEntity;
import org.colinvella.fancyfish.entity.fish.YellowTangEntity;
import org.colinvella.fancyfish.logger.ModLogger;

import net.minecraft.entity.EnumCreatureType;
import net.minecraft.world.biome.BiomeGenBase;
import net.minecraftforge.fml.common.registry.EntityRegistry;

public class ModEntityRegistry
{
    public static List<Class<? extends FishEntity>> getModFishEntityClasses()
    {
        return Collections.unmodifiableList(modFishEntityClasses);
    }

    public static void registerEntities()
    {
        logger = FancyFishMod.getLogger();
        logger.info("Registering entities...");
        ModEntityRegistry.registerFishEntity(ClownFishEntity.class);
        ModEntityRegistry.registerFishEntity(RegalTangEntity.class);
        ModEntityRegistry.registerFishEntity(MoorishIdolEntity.class);
        ModEntityRegistry.registerFishEntity(RoyalGrammaEntity.class);
        ModEntityRegistry.registerFishEntity(PufferFishEntity.class);
        ModEntityRegistry.registerFishEntity(YellowTangEntity.class);
        ModEntityRegistry.registerFishEntity(DamselFishEntity.class);
        ModEntityRegistry.registerFishEntity(BlennyEntity.class);
        ModEntityRegistry.registerFishEntity(MoonFishEntity.class);
    }

    private static void registerFishEntity(Class<? extends FishEntity> fishEntityClass)
    {
        String fishEntityId = (String)ModEntityRegistry.getStaticFieldValue(fishEntityClass, "ID", String.class);
        if (fishEntityId == null)
        {
            logger.error("Unable to register fish entity class " + fishEntityClass.getName());
            return;
        }
        
        Object primaryEggColourObject = ModEntityRegistry.getStaticFieldValue(fishEntityClass, "PRIMARY_EGG_COLOUR", Integer.class);
        if (primaryEggColourObject == null)
        {
            logger.error("Unable to register fish entity class " + fishEntityClass.getName());
            return;
        }
        
        Object secondaryEggColourObject = ModEntityRegistry.getStaticFieldValue(fishEntityClass, "SECONDARY_EGG_COLOUR", Integer.class);
        if (secondaryEggColourObject == null)
        {
            logger.error("Unable to register fish entity class " + fishEntityClass.getName());
            return;
        }
        
        int primaryEggColour = (Integer)primaryEggColourObject;
        int secondaryEggcolour = (Integer)secondaryEggColourObject;
        
        int entityIndex = ModEntityRegistry.getNextEntityIndex();
        
        logger.info("Registering fish entity " + fishEntityId + "...");
        logger.debug("Registering fish entity class " + fishEntityId + " with entity index " + entityIndex + "...");
        EntityRegistry.registerModEntity(fishEntityClass, (String)fishEntityId, (int)entityIndex, (Object)FancyFishMod.instance, 80, 3, true);
        
        logger.debug("Registering fish entity spawn egg for " + fishEntityId + " with colours #" + Integer.toHexString(primaryEggColour) + ", #" + Integer.toHexString(secondaryEggcolour) + "...");
        EntityRegistry.registerEgg(fishEntityClass, (int)primaryEggColour, (int)secondaryEggcolour);
        
        logger.debug("Enabling spawn capability for fish entity " + fishEntityId + " in water biomes...");
		EntityRegistry.addSpawn(fishEntityClass, 50, 4, 1024, EnumCreatureType.WATER_CREATURE,
				BiomeGenBase.ocean, BiomeGenBase.beach, BiomeGenBase.deepOcean, BiomeGenBase.river);

		// track entity classes
        modFishEntityClasses.add(fishEntityClass);
    }

    private static int getNextEntityIndex()
    {
        return nextEntityIndex++;
    }

    private static Object getStaticFieldValue(Class klass, String fieldName, Class expectedType)
    {
        Field idField = null;
        try
        {
            idField = klass.getField(fieldName);
        }
        catch (Exception exception)
        {
            logger.error("Unable to extract static field " + fieldName + " from class " + klass.getTypeName() + ". Reason: " + exception.getMessage());
            return null;
        }
        
        Object value = null;
        try {
            value = idField.get(null);
        }
        catch (Exception exception) {
            logger.error("Unable to extract value of static field " + fieldName + " from class " + klass.getTypeName() + ". Reason: " + exception.getMessage());
            return null;
        }
        
        if (value.getClass() != expectedType)
        {
            logger.error("The field " + fieldName + " of class " + klass.getTypeName() + " is not of the expected type " + expectedType.getTypeName());
            return null;
        }
        return value;
    }

    private static ModLogger logger;
    private static int nextEntityIndex = 0;
    private static List<Class<? extends FishEntity>> modFishEntityClasses = new ArrayList<Class<? extends FishEntity>>();
}

