package org.colinvella.fancyfish.item.combat;

import org.colinvella.fancyfish.achievement.ModAchievements;

import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.boss.EntityDragon;
import net.minecraft.entity.monster.EntityBlaze;
import net.minecraft.entity.monster.EntityGhast;
import net.minecraft.entity.monster.EntityMagmaCube;
import net.minecraft.entity.monster.EntityWitch;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.potion.Potion;
import net.minecraft.potion.PotionEffect;
import net.minecraft.world.World;
import net.minecraftforge.common.util.EnumHelper;

public class FishScaleSwordItem extends ModItemSword
{
	public static final String ID = "FishScaleSword";
	
	public FishScaleSwordItem()
	{
		super(ID, Material);
	}

	// adds 1 extra point of damage if mob is harmed by water
	@Override
	public boolean hitEntity(ItemStack stack, EntityLivingBase target, EntityLivingBase attacker)
	{
		if (isHydrophobicMob(target))
		{
	        stack.damageItem(2, attacker);
	        
	        World world = target.worldObj;
	        target.addPotionEffect(new PotionEffect(Potion.wither.id, 20, 4));
	        
	        return true;			
		}
		else
			return super.hitEntity(stack, target, attacker);
	}
	
	@Override
	public boolean hasEffect(ItemStack stack)
	{
		// shimmers
		return true;
	};
	
	// trigger achievement for crafting sword
	@Override
	public void onCreated(ItemStack stack, World worldIn, EntityPlayer playerIn)
	{
		playerIn.addStat(ModAchievements.BuildFishScaleSword, 1);
	}
	
	private boolean isHydrophobicMob(EntityLivingBase target)
	{
		return target instanceof EntityBlaze
			|| target instanceof EntityGhast
			|| target instanceof EntityMagmaCube
			|| target instanceof EntityWitch
			|| target instanceof EntityDragon;
	}
	
	private static final int HarvestLevel = 0;
	private static final int Durability = 1200;
	private static final int MiningSpeed = 1;
	private static final int DamageVsEntities = 1;
	private static final int Enchantability = 25;

	private static final ToolMaterial Material = EnumHelper.addToolMaterial(
		"FishScale", HarvestLevel, Durability, MiningSpeed,
		DamageVsEntities, Enchantability);

}
