package org.colinvella.fancyfish.item.tool;

import org.colinvella.fancyfish.achievement.ModAchievements;

import net.minecraft.block.Block;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.util.EnumHelper;

public class FishScaleAxeItem extends ModItemAxe
{
	public static final String ID = "FishScaleAxe";
	
	public FishScaleAxeItem()
	{
		super(ID, Material);
	}
	
	
	@Override
	public boolean hasEffect(ItemStack stack)
	{
		// shimmers
		return true;
	};
	
	// trigger achievement for crafting axe
	@Override
	public void onCreated(ItemStack stack, World worldIn, EntityPlayer playerIn)
	{
		playerIn.addStat(ModAchievements.BuildFishScaleAxe, 1);
	}
	
	@Override
	public boolean onBlockDestroyed(ItemStack stack, World world, Block block, BlockPos pos, EntityLivingBase player)
	{
		// only on server side
		if (world.isRemote)
			return super.onBlockDestroyed(stack, world, block, pos, player);

		// custom behaviour only for logs
		if (block != Blocks.log && block != Blocks.log2)
			return super.onBlockDestroyed(stack, world, block, pos, player);

		// determine item drop randomly
		int randomItem = this.itemRand.nextInt(20);
		
		Item itemDrop = null;
		String soundId = null;
		
		if (randomItem == 19)
		{
			// 5% chance golden apple
			itemDrop = Items.golden_apple;
			soundId = "random.levelup";
		}
		else if (randomItem > 9)
		{
			// 45% chance apple
			itemDrop = Items.apple;
			soundId = "random.orb";
		}

		if (itemDrop != null)
		{
			double midX = pos.getX() + 0.5;
			double midY = pos.getY() + 0.5;
			double midZ = pos.getZ() + 0.5;
			world.spawnEntityInWorld(new EntityItem(world, midX, midY, midZ,
				new ItemStack(itemDrop)));		
			
			world.playSoundEffect(midX, midY, midZ, soundId, 1.0F, 0.8f);
		}
		
		return super.onBlockDestroyed(stack, world, block, pos, player);
	};
	
	private static final int HarvestLevel = 0;
	private static final int Durability = 1200;
	private static final float MiningSpeed = 8.0f;
	private static final int DamageVsEntities = 1;
	private static final int Enchantability = 25;
	
	private static final ToolMaterial Material = EnumHelper.addToolMaterial(
			"FishScale", HarvestLevel, Durability, MiningSpeed,
			DamageVsEntities, Enchantability);
}
