package org.colinvella.fancyfish.item.tool;

import org.colinvella.fancyfish.achievement.ModAchievements;
import org.colinvella.fancyfish.creativetab.ModCreativeTabs;
import org.colinvella.fancyfish.entity.fish.FishEntity;
import org.colinvella.fancyfish.item.CapturedFishItem;
import org.colinvella.fancyfish.item.ModItem;
import org.colinvella.fancyfish.item.ModItemRegistry;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.world.World;

public class FishingNetItem extends ModItem
{
	public static final String ID = "FishingNet";
	
	public FishingNetItem()
	{
		super(ID, 1, ModCreativeTabs.FancyFishTab);

		// assume always made of wood
		toolMaterial = ToolMaterial.WOOD;
		setMaxDamage(toolMaterial.getMaxUses());
	}
		
	@Override
	public boolean onLeftClickEntity(ItemStack itemStack, EntityPlayer entityPlayer, Entity entity) 
	{
		World world = entityPlayer.worldObj;

		// if target entity is not a fish, apply default behaviour (hit block, entity etc.)
		if (!(entity instanceof FishEntity))
			return super.onLeftClickEntity(itemStack, entityPlayer, entity);
		
		if (!entityPlayer.inventory.consumeInventoryItem(ModItemRegistry.FishBowl))
		{
			if (!world.isRemote)
			{
				entityPlayer.addChatMessage(new ChatComponentTranslation("tile.FishTank.message.needEmptyBowl"));
			}
			return true;
		}
		
		// cast entity to fish and get corresponding fish in a bowl item
		FishEntity fishEntity = (FishEntity) entity;
		CapturedFishItem capturedFishItem = fishEntity.getCapturedFishItem();

		ItemStack capturedFishItemStack = new ItemStack(capturedFishItem);
		
		// store fish's custom name tag in item stack for later assignment (my daughter likes to name her fish!)
		NBTTagCompound nbtTagCompound = new NBTTagCompound();
		nbtTagCompound.setString("tag", fishEntity.getCustomNameTag());
		capturedFishItemStack.setTagCompound(nbtTagCompound);
		
		// add the appropriate fish in bowl item to the inventory
		entityPlayer.inventory.addItemStackToInventory(capturedFishItemStack);
		
		// remove the entity from the world
		world.removeEntity(entity);
		
		// apply damage to fishing net
		itemStack.damageItem(1, entityPlayer);
		
		// add achievement
		entityPlayer.addStat(ModAchievements.CatchFish, 1);
		
		logger.debug("Player captured a fish entity of type: " + fishEntity.getClass().getSimpleName());
		
		return true;
	}
	
	
	// trigger achievement for crafting fishing net
	@Override
	public void onCreated(ItemStack stack, World worldIn, EntityPlayer playerIn)
	{
		playerIn.addStat(ModAchievements.BuildFishingNet, 1);
	}
	
	private ToolMaterial toolMaterial;
}
