package org.colinvella.fancyfish.tileentity;

import java.util.Random;

public class CapturedFishState
{
	public CapturedFishState()
	{
		this.random = new Random();
		
		this.posX = 0.25 + random.nextDouble() * 0.5;
		this.posY = 0.25 + random.nextDouble() * 0.45;
		this.posZ = 0.25 + random.nextDouble() * 0.5;
		
		this.setNewTarget();
	}
	
	public void update(double deltaTime)
	{		
		double deltaX = this.targetX - this.posX;
		double deltaY = this.targetY - this.posY;
		double deltaZ = this.targetZ - this.posZ;
		
		double delta = Math.sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);
		
		if (delta < 0.3)
		{
			this.setNewTarget();
		}
		
		double accX = deltaX * 2.0;
		double accY = deltaY * 2.0;
		double accZ = deltaZ * 2.0;
		
		this.velX += accX * deltaTime;
		this.velY += accY * deltaTime;
		this.velZ += accZ * deltaTime;
				
		this.posX += this.velX * deltaTime;
		this.posY += this.velY * deltaTime;
		this.posZ += this.velZ * deltaTime;

		this.posX = clamp(this.posX, 0.25, 0.75);
		this.posY = clamp(this.posY, 0.25, 0.70);
		this.posZ = clamp(this.posZ, 0.25, 0.75);
	}
	
	public double getYaw()
	{
		return Math.atan2(this.velX, this.velZ) * RAD_TO_DEG;
	}

	public double getPitch()
	{
		double xzSpeed = Math.sqrt(this.velX * this.velX + this.velZ * this.velZ);
		return Math.atan2(this.velY, xzSpeed) * RAD_TO_DEG;
	}

	
	private void setNewTarget()
	{
		this.targetX = 0.25 + random.nextDouble() * 0.5;
		this.targetY = 0.25 + random.nextDouble() * 0.45;
		this.targetZ = 0.25 + random.nextDouble() * 0.5;
	}
	
	private double clamp(double value, double min, double max)
	{
		value = Math.max(min, value);
		value = Math.min(max, value);
		return value;
	}
	
	public double posX;
	public double posY;
	public double posZ;
	public double velX;
	public double velY;
	public double velZ;
	public double targetX;
	public double targetY;
	public double targetZ;
	
	private Random random;

	private static final double MAX_ACCEL = 0.01;
	private static final double MAX_SPEED = 0.02;
	private static final double RAD_TO_DEG =180.0 / Math.PI ; 
}
