package org.colinvella.fancyfish.world;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.colinvella.fancyfish.FancyFishMod;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.util.BlockPos;
import net.minecraft.world.World;
import net.minecraft.world.WorldSavedData;
import net.minecraftforge.common.util.Constants;

public class ModSaveData extends WorldSavedData
{
	public ModSaveData()
	{
		super(ID);
		this.teleportPositions = new ArrayList<BlockPos>();
	}
	
	// needed for deserialiser
	public ModSaveData(String identifier)
	{
		super(identifier);
		this.teleportPositions = new ArrayList<BlockPos>();
	}
	
	@Override
	public void readFromNBT(NBTTagCompound nbt)
	{
		teleportPositions.clear();
		
		NBTTagList nbtTagList = nbt.getTagList("TeleportPositions", Constants.NBT.TAG_COMPOUND);
		if (nbtTagList == null)
			return;
		
		for (int index = 0; index < nbtTagList.tagCount(); index++)
		{
			NBTTagCompound nbtTagCompound = (NBTTagCompound) nbtTagList.get(index);
			if (nbtTagCompound == null)
				continue;
			
			int posX = nbtTagCompound.getInteger("X");
			int posY = nbtTagCompound.getInteger("Y");
			int posZ = nbtTagCompound.getInteger("Z");
			
			teleportPositions.add(new BlockPos(posX, posY, posZ));
		}
	}

	@Override
	public void writeToNBT(NBTTagCompound nbt)
	{
		NBTTagList nbtTagList = new NBTTagList();
		for(BlockPos pos: teleportPositions)
		{
			NBTTagCompound nbtTagCompound = new NBTTagCompound();
			nbtTagCompound.setInteger("X", pos.getX());
			nbtTagCompound.setInteger("Y", pos.getY());
			nbtTagCompound.setInteger("Z", pos.getZ());
			nbtTagList.appendTag(nbtTagCompound);
		}
		
		nbt.setTag("TeleportPositions", nbtTagList);
	}
	
	public List<BlockPos> getTeleportPositions()
	{
		return Collections.unmodifiableList(teleportPositions);
	}
	
	public boolean registerTeleportPosition(World world, BlockPos pos)
	{
		if (teleportPositions.indexOf(pos) >= 0)
			return false;
		teleportPositions.add(pos);
		world.setItemData(ID, this);
		markDirty();
		world.getMapStorage().saveAllData();
		return true;
	}

	public boolean deregisterTeleportPosition(World world, BlockPos pos)
	{
		if (teleportPositions.indexOf(pos) < 0)
			return false;
		teleportPositions.remove(pos);
		world.setItemData(ID, this);
		markDirty();
		world.getMapStorage().saveAllData();
		return true;
	}

	public static ModSaveData get(World world)
	{
		ModSaveData modSaveData = (ModSaveData)world.loadItemData(ModSaveData.class, ID);
		if (modSaveData == null)
		{
			modSaveData = new ModSaveData();
			world.setItemData(ID, modSaveData);
			modSaveData.markDirty();
			world.getMapStorage().saveAllData();
		}
		return modSaveData;
	}
	
	private List<BlockPos> teleportPositions;
	
	private static final String ID = FancyFishMod.MOD_ID + "SaveData";
}